#!/usr/bin/python3
"""
Preparation of the partitioning of jsons about audio data

Author: Soline Felice, Laboratoire d'Informatique de Grenoble

Date: 19/10/2022

Goal : Split the metadata about the audios from audiocite into 3 jsons : dev (10% of total duration), test (10% of total duration) and train (80% of total duration).

Input :
- ./metadata/all_filtered_files.json (all_files.json without the audios whose category are "charme", "planete-actuelle" and "religion")
- ./metadata/all_files.json (the full json file (about all the audios))

Treatment : 
- Open and read the json file filtered * ("all_filtered_files.json")
    - Get 350h of speaker M and 350h of speaker F (but only data where the gender and id of the speaker are filled in (not files with "U"))
    - Put it in a json file "dev" 
    - Write and put the data on a json file "dev"
- Open and read the json file filtered * ("all_filtered_files.json")
    - Get 350h of speaker M and 350h of speaker F (with the same criteria than the "dev" but the following data)
    - Put it in a json file "test" 
    - Write and put the data on a json file "test" 
- Open and read the complete json file ("all_files.json")
    - Get all the data which aren't allready in "dev" or "test"
    - Write and put the data on a json file "train"

* NB : "json file filtered" means here the json about all wav file without :
    - the files whose category is "charme", "planete-actuelle" and "religion"
    - the files whose id or gender is "U" (unkown)

Output :
- ./metadata/dev_files.json (json dev file)
- ./metadata/test_files.json (json test file)
- ./metadata/train_files.json (json train file)

Prerequisites :
- Libraries installed :
    - pydub

- Script you need to have run before :
    - prep_json_metadata.py (to have the "all_files.json")
    - prep_json_filter.py (to have the "all_filtered_files.json")

- Architecture (folders and files necessary):
    - ./scripts/
        - prep_json_partitioning.py (this script)
        - funcs.py (necessary)

    - ./metadata/
        - ids_gender.txt (file with the informations of the speakers and their genders in format "gender\tname", ex : F\tCarole \n M\tGérard))
        - downloaded.csv (informations about the audios extract at the same time)
        - all_files.json
        - all_filtered_files.json

    - ./wavs/
        - (wavs folders with the audios files in it)


"""
import json
import os

########## Import of files ##########

# Filtered json file throught iterate jsons dev and test
json_without_charmes_actu_religion = ("../metadata/all_filtered_files.json")

# Full json file throught iterate json train (all the others wavs than dev and test)
full_json_file = ("../metadata/all_files.json")

# Output file for dev
json_dev_file = ("../metadata/dev_files.json")

# Output file for test
json_test_file = ("../metadata/test_files.json")

# Output file for train
json_train_file = ("../metadata/train_files.json")

#####################################

class CreateJsonFiles():

    def __init__(self) :

        self.filename = None
        self.devFile = None
        self.testFile = None
        self.trainFile = None
        self.cpt_dev_female_duration = None
        self.cpt_dev_male_duration = None
        self.cpt_test_male_duration = None
        self.cpt_test_female_duration = None
        self.cpt_train_duration = None
        self.i = None


    def write_json_dev(self):

        with open(json_without_charmes_actu_religion, "r") as json_file_filtered, open (json_dev_file, "w") as jsonDevFile   :

            self.cpt_dev_male_duration = float(0)
            self.cpt_dev_female_duration = float(0)
            limit_female_duration = float(1260000)
            limit_male_duration = float(1260000)

            # Reading the json file
            try:
                self.devFile = json.load(jsonDevFile)
            except:
                self.devFile = {}
                json.dump(self.devFile, jsonDevFile)

            data_file = json.load(json_file_filtered)

            for i in data_file :
                spk_gender = data_file[i]['spk_gender']
                # wavs_name = data_file[i]
                wav_duration = data_file[i]['duration']
                if spk_gender == "U" :
                    pass
                elif spk_gender == "F" :
                    while self.cpt_dev_female_duration <= limit_female_duration : # While the counter is under 350h
                        self.cpt_dev_female_duration += wav_duration
                        self.devFile[i] = data_file[i]
                        # print ("cpt dev female duration : ", cpt_dev_female_duration)
                        wav_name_female_stop_dev = i
                        #print("Wav name female where dev stop : ",wav_name_female_stop_dev)
                        break
                        return wav_name_female_stop_dev, wav_duration, wavs_name

                elif spk_gender == "M" :
                    while self.cpt_dev_male_duration <= limit_male_duration : # While the counter is under 350h
                        self.cpt_dev_male_duration += wav_duration
                        self.devFile[i] = data_file[i]
                        #print ("cpt dev male duration : ", cpt_dev_male_duration)
                        wav_name_male_stop_dev = i
                        #print("Wav name male where dev stop : ", wav_name_male_stop_dev)
                        break
                        return wav_name_male_stop_dev, wav_duration, wavs_name

    ####################################################################################################################
    
    def write_json_test(self):

        with open(json_without_charmes_actu_religion, "r") as json_file_filtered, open (json_test_file, "w") as jsonTestFile :

            self.cpt_test_male_duration = float(0)
            self.cpt_test_female_duration = float(0)

            # Reading the json file
            try:
                self.testFile = json.load(jsonTestFile)
            except:
                self.testFile = {}
                json.dump(self.testFile, jsonTestFile)

            data_file = json.load(json_file_filtered)
            #data_test_file.popitem(data_dev_file)

            sum_wav_female_duration = float(0)
            sum_wav_male_duration = float(0)
            limit_female_duration = float(1260000)
            limit_male_duration = float(1260000)

            for i in data_file :
                spk_gender = data_file[i]['spk_gender']
                wav_duration = data_file[i]['duration']
                wavs_name = data_file[i]
                if spk_gender == "F" :
                    sum_wav_female_duration += wav_duration 
                    if sum_wav_female_duration > self.cpt_dev_female_duration :
                        data_file_test = i
                        while self.cpt_test_female_duration <= limit_female_duration : #1260000 While the counter is low than 350h
                            self.cpt_test_female_duration += wav_duration
                            self.testFile[i] = data_file[i]
                            devFileKeys = self.devFile.keys()
                            if i not in devFileKeys :
                                pass
                            else :
                                print ("Caution : The test file ",i," is already in the dev partition !")
                            break

                elif spk_gender == "M" :
                    sum_wav_male_duration += wav_duration 
                    if sum_wav_male_duration > self.cpt_dev_male_duration :
                        data_file_test = i
                        while self.cpt_test_male_duration <= limit_male_duration : #1260000 While the counter is low than 350h
                            devFileKeys = self.devFile.keys()
                            if i not in devFileKeys :
                                self.cpt_test_male_duration += wav_duration
                                self.testFile[i] = data_file[i]
                            else :
                                print ("Caution : The test file ",i," is already in the dev partition !")
                            break
                elif spk_gender == "U" :
                    pass

    ####################################################################################################################

    def write_json_train(self):
            
        with open(full_json_file, "r") as complete_json_file, open (json_train_file, "w") as jsonTrainFile   :

            self.cpt_train_duration = float(0)

            # Reading the json file
            try:
                self.trainFile = json.load(jsonTrainFile)
            except:
                self.trainFile = {}
                json.dump(self.trainFile, jsonTrainFile)


            full_data_file = json.load(complete_json_file)

            for i in full_data_file :
                wav_duration = full_data_file[i]['duration']
                fullFile = full_data_file.items()
                for wavName,wavInfos in fullFile :                  
                    if i in self.devFile :
                        # print("wav name dans devfile : ", wavName) 
                        pass
                    elif i in self.testFile :
                        # print("wav name dans testfile : ", wavName)
                        pass
                    elif not i in self.devFile and not i in self.testFile :
                        if i in self.trainFile :
                            break 
                        else :
                            self.cpt_train_duration += wav_duration
                            self.trainFile[i] = full_data_file[i]
                            
    
    def create_dict_to_dump(self, new_entry): 
        """! Warning: if two files have the same name, they won't be added to the dict twice. The second will replace the first. The if statement is needed."""   

        if self.filename in self.devFile:
            self.filename = self.filename+'_'+str(self.i)
            self.devFile[self.filename] = new_entry 
            # return self.devFile
        if self.filename in self.testFile:
            self.filename = self.filename+'_'+str(self.i)
            self.testFile[self.filename] = new_entry 
            # return testFile
        if self.filename in self.trainFile:
            self.filename = self.filename+'_'+str(self.i)
            self.trainFile[self.filename] = new_entry 
            # return trainFile


    def write_json_file(self):
        # write dict and dump. Creates multiple json (mandatory because dumping dict too large fails and data needs to be written once
        # or it creates multiple json objects and leads to a badly formed json file)

        with open(json_dev_file, "w") as jsonDevFile:
            json.dump(self.devFile, jsonDevFile, ensure_ascii=False, indent=2, separators=(',', ': '))

        with open(json_test_file, "w") as jsonTestFile:
            json.dump(self.testFile, jsonTestFile, ensure_ascii=False, indent=2, separators=(',', ': '))

        with open(json_train_file, "w") as jsonTrainFile:
            json.dump(self.trainFile, jsonTrainFile, ensure_ascii=False, indent=2, separators=(',', ': '))

        #erase json_data
        self.devFile.clear() 
        self.testFile.clear() 
        self.trainFile.clear() 

    def getFile(self) :
        return self.devFile, self.testFile, self.trainFile


def main():

    entry_num=0
    json_file_num=1

    new_entry = {}

    json_file = CreateJsonFiles()
    json_file.write_json_dev()
    json_file.write_json_test()
    json_file.write_json_train()

    if  entry_num <=40000: #sets number of json entries in json file. If too many, it crashes. Default = 40 000
        dict_to_dump = json_file.create_dict_to_dump(new_entry)
        entry_num+=1

    else:
        dict_to_dump = json_file.create_dict_to_dump(new_entry)
        json_file.write_json_dev()
        json_file.write_json_test()
        json_file.write_json_train()
        entry_num=0
        json_file_num+=1

    json_file_num+=1
    json_file.write_json_dev()
    json_file.write_json_test()
    json_file.write_json_train()
    

    devFile, testFile, trainFile = json_file.getFile()
    #json_file.write_final_CSV()

    print("\nJson Dev file is being written \n")
    with open(json_dev_file, "w") as jsonDevFile:
        json.dump(devFile, jsonDevFile, ensure_ascii=False, indent=2, separators=(',', ': '))

    print("\nJson Test file is being written \n")
    with open(json_test_file, "w") as jsonTestFile:
        json.dump(testFile, jsonTestFile, ensure_ascii=False, indent=2, separators=(',', ': '))

    print("\nJson Train file is being written \n")
    with open(json_train_file, "w") as jsonTrainFile:
        json.dump(trainFile, jsonTrainFile, ensure_ascii=False, indent=2, separators=(',', ': '))


if __name__== "__main__":

    main()  
